//******************************************************************************************
//* Hyperbolic Dodecahedron with Spherical Faces, by Paul Nylander, bugman123.com, 2/15/07
//* runtime: 1 second
//******************************************************************************************

#macro Sqr(X) X*X #end
#declare zface=sqrt(0.55+0.9/sqrt(5)); #declare rface=sqrt(0.6-0.1/sqrt(5));
#declare Alpha=acos(-sqrt(5)/5); // 116.56505117707799 // flat dihedral angle

//see Matthias Weber's page: http://www.math.uni-bonn.de/people/weber/research/hyperbolic/index.html
//#declare z0=-1e-10; // Euclidean, dihedral: Alpha
//#declare z0=-0.15253740538096178*rface; // ideal hyperbolic, dihedral: 90
//#declare z0=-0.28374460792490186*rface; // ideal hyperbolic, dihedral: 72
#declare z0=-0.47864212753950064*rface; // ideal hyperbolic, dihedral: 60 (vertices at infinity)

camera{location <0,-16,0> look_at <0,0,0> sky y up y right x*image_width/image_height angle 10}
light_source{<0,-16,0>,1}

#macro hue(h)
 #local grad=array[7] {<0,1,0,0>,<1/6,1,1,0>,<1/3,0,1,0>,<1/2,0,1,1>,<2/3,0,0,1>,<5/6,1,0,1>,<1,1,0,0>};
 #local n=dimension_size(grad,1); #local i=0;
 #while(i<n & grad[min(i,n-1)].x<h) #local i=i+1; #end
 #if(i>0 & i<n) #local x1=grad[i-1].x; #local x2=grad[i].x; #local c=((x2-h)*grad[i-1]+(h-x1)*grad[i])/(x2-x1); #else #local c=grad[min(i,n-1)]; #end
 <c.y,c.z,c.t>
#end

#declare Seed=seed(10);
#macro face(phi,theta) sphere{<0,0,zface-(Sqr(rface)/z0-z0)/2>,(Sqr(rface)/z0+z0)/2 pigment{rgb hue(rand(Seed))}
 rotate <0,degrees(phi),degrees(theta-pi/10)>}
#end
difference{sphere{<0,0,0>,sqrt(Sqr(rface)+Sqr(zface))} face(0,0) face(pi,0) #declare theta=0;
 #while(theta<2*pi) face(Alpha,theta) face(pi-Alpha,theta+pi/5) #declare theta=theta+2*pi/5; #end
}
